from RSTAB.initModel import Model

class NodalRelease():
    def __init__(self,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        pass
        # TODO: Nodal Release missing in WS Core